﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Labs
{
    public partial class MailTextBox : UserControl
    {
        public MailTextBox()
        {
            InitializeComponent();
            Template = @"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$";
        }

        //пример корректного ввода для ToolTip
        private string example = "something@gmail.com";

        public void setExample(string example)
        {
            Regex regex = new Regex(Template);
            if (regex.IsMatch(example))
            {
                this.example = example;
            }
        }

        //шаблон поля ввода
        private string template;

        //св-во для работы с шаблоном поля ввода
        public string? Template
        {
            get { return template; }
            set { template = value; }
        }

        //св-во для введённого значения в textbox
        public string TextValue
        {
            get
            {
                //если не задан шаблон
                if (Template == null)
                {
                    throw new TemplateIsNotDefinedException();
                }
                Regex regex = new Regex(Template);
                //если не соответствует шаблону
                if (!regex.IsMatch(textBox.Text))
                {
                    throw new DoesNotMatchTemplateException();
                }
                return textBox.Text;
            }

            set
            {
                //если задан шаблон
                if (Template != null)
                {
                    Regex regex = new Regex(Template);
                    //если соответствует шаблону
                    if (regex.IsMatch(value))
                    {
                        textBox.Text = value;
                    }
                }
                //если не задан шаблон, ничего не вывести
                else
                {
                    textBox.Text = "";
                }
            }
        }

        private class TemplateIsNotDefinedException : Exception
        {
            public TemplateIsNotDefinedException() : base("Шаблон текста не задан") { }
        }

        private class DoesNotMatchTemplateException : Exception
        {
            public DoesNotMatchTemplateException() : base("Текст не соответствует шаблону") { }
        }

        private void textBox_Enter(object sender, EventArgs e)
        {
            ToolTip tooltip = new ToolTip();
            tooltip.Show(example, textBox, 0, -30, 2500);
        }


        private EventHandler onTextChanged;
        public event EventHandler TextChanged
        {
            add
            {
                onTextChanged += value;
            }
            remove
            {
                onTextChanged -= value;
            }
        }

        private void textBox_TextChanged(object sender, EventArgs e)
        {
            onTextChanged?.Invoke(sender, e);
        }
    }
}
